/**
 * 
 */
package org.springblade.cube.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.cube.dto.CubeDataPermRelation;
import org.springblade.cube.mapper.CubeDataPermissionMapper;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeDataPermission;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.CubePermParseDto;
import org.springblade.visual.dto.CubePermParseDto.CubePermParseDtoBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.util.StrUtil;

/**
 * @author wookvn
 *
 */
@Service
public class CubeDataPermissionService extends ServiceImpl<CubeDataPermissionMapper, CubeDataPermission> {
	
	@Autowired
	private DataPermClient dataPermClient;
	
	@Autowired
	private CubeDataService cubeDataService;
	
	/**
	 * 
	 * @param cubeIdList
	 * @return
	 */
	public List<CubeDataPermRelation> selectListByCubeIdList(List<Integer> cubeIdList){
		return baseMapper.selectListByCubeIdList(cubeIdList);
	}
	
	/**
	 * 根据权限组ID和分类ID查询
	 * @param dataPermId
	 * @param cubeWorkspaceId
	 * @return
	 */
	public List<CubeDataPermission> selectListByDataPermIdAndWorkspaceId(Integer dataPermId, Integer cubeWorkspaceId){
		return baseMapper.selectListByDataPermIdAndWorkspaceId(dataPermId, cubeWorkspaceId);
	}
	
	/**
	 * 检查用户是否有数据集的访问权限
	 * @param cubeDataId
	 * @param cubeCreateUserId
	 * @param userId
	 * @return
	 */
	public boolean checkCubePermForUser(Integer cubeDataId, Long cubeCreateUserId, BladeUser bladeUser) {
		//检查4个方面：
		//1.当前用户是否创建人
		//2.是否公开
		//3.是否超级权限组
		//4.当前用户是否拥有权限
		return cubeCreateUserId.longValue() == bladeUser.getUserId()
				|| cubeDataService.count(Wrappers.<CubeData>lambdaQuery().eq(CubeData::getId, cubeDataId).eq(CubeData::getPublicStatus, 1)) > 0
				|| Optional.<Boolean>ofNullable(dataPermClient.checkSuperPermission(bladeUser.getUserId())).orElse(false)
				|| CollectionUtil.isNotEmpty(baseMapper.selectListByCubeDataIdAndUserId(cubeDataId, bladeUser.getUserId()));
	}
	
	/**
	 * 从权限组管理入口添加
	 * @param dataPermId
	 * @param cubeWorkspaceId
	 * @param userId
	 * @param dataIdList
	 * @return
	 */
	@Transactional
	public boolean doBatchSubmit(Integer dataPermId, Integer cubeWorkspaceId, Long userId, List<Integer> dataIdList) {
		//1.查询某个权限组下，某个数据集分类的所有数据
//		List<CubeDataPermission> dataList = baseMapper.selectList(Wrappers.<CubeDataPermission>lambdaQuery().eq(CubeDataPermission::getDataPermId, dataPermId).eq(CubeDataPermission::getCubeWorkspaceId, cubeWorkspaceId));
		List<CubeDataPermission> dataList = selectListByDataPermIdAndWorkspaceId(dataPermId, cubeWorkspaceId);
		List<Integer> dbCubeIdList = new ArrayList<>();
		if(CollectionUtil.isNotEmpty(dataList)) {
			dataList.stream().map(CubeDataPermission::getCubeDataId).forEach(dbCubeIdList::add);
		}
		//2.获取删除的和添加的数据列表
		List<Integer> addCubeIdList = dataIdList.stream().filter(t -> !dbCubeIdList.contains(t)).collect(Collectors.toList());
		List<Integer> delCubeIdList = dbCubeIdList.stream().filter(t -> !dataIdList.contains(t)).collect(Collectors.toList());
		if(CollectionUtil.isNotEmpty(addCubeIdList)) {
			//根据ID更新数据集的公开状态为0(不公开)
			cubeDataService.update(Wrappers.<CubeData>lambdaUpdate().set(CubeData::getPublicStatus, 0).in(CubeData::getId, addCubeIdList));
			List<CubeDataPermission> entityList = new ArrayList<>();
			for(Integer cubeDataId : addCubeIdList) {
				CubeDataPermission cubeDataPerm = new CubeDataPermission();
				cubeDataPerm.setCreateUserId(userId);
				cubeDataPerm.setCreateTime(new Date());
				cubeDataPerm.setCubeDataId(cubeDataId);
//				cubeDataPerm.setCubeWorkspaceId(cubeWorkspaceId);
				cubeDataPerm.setCubePermName("过滤器"+cubeDataId);
				cubeDataPerm.setDataPermId(dataPermId);
				entityList.add(cubeDataPerm);
			}
			saveBatch(entityList);
		}
		if(CollectionUtil.isNotEmpty(delCubeIdList)) {
			remove(Wrappers.<CubeDataPermission>lambdaQuery().eq(CubeDataPermission::getDataPermId, dataPermId).in(CubeDataPermission::getCubeDataId, delCubeIdList));
		}
		return true;
	}
	
	/**
	 * 
	 * @param cubeDataId
	 * @param bladeUser
	 * @return
	 */
	public CubePermParseDto parseCubePermission(Integer cubeDataId, BladeUser bladeUser) {
		CubePermParseDtoBuilder parseDtoBuilder = CubePermParseDto.builder();
		if(bladeUser == null) { //智能预警，用户为Null，兼容处理
			return parseDtoBuilder.build();
		}
		List<CubeDataPermission> cubeDataPermList = baseMapper.selectListByCubeDataIdAndUserId(cubeDataId, bladeUser.getUserId());
		if(CollectionUtil.isNotEmpty(cubeDataPermList)) {
			List<String> whereSqlList = new ArrayList<>();
			Set<Integer> excludeColumnIdSet = new HashSet<>();
			for(CubeDataPermission cubePermission : cubeDataPermList) {
				if(StrUtil.isNotEmpty(cubePermission.getFilterSql())) {
					whereSqlList.add("(" + cubePermission.getFilterSql() + ")");
				}
				if(StrUtil.isNotEmpty(cubePermission.getExcludeColumnIds())) {
					String[] _columnIds = cubePermission.getExcludeColumnIds().split(",|，");
					excludeColumnIdSet.addAll(Stream.of(_columnIds).map(Integer::valueOf).collect(Collectors.toSet()));
				}
			}
			//多个权限组之间的关系使用OR
			parseDtoBuilder.filterSql(String.join(" OR ", whereSqlList)).excludeColumnIdList(excludeColumnIdSet);
		}
		return parseDtoBuilder.build();
	}
}
