/**
 *
 */
package org.springblade.cube.service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.dto.CubeStaticDataParam;
import org.springblade.cube.dto.TemporaryCubeDataParam;
import org.springblade.cube.enums.AggregationTypeEnum;
import org.springblade.cube.enums.ColumnAttrEnum;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.cube.enums.TableTypeEnum;
import org.springblade.cube.mapper.CubeTableStaticMapper;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.model.CubeTableStatic;
import org.springblade.cube.model.FileStructure;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.cube.utils.SnowFlowIntUtil;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 */
@Slf4j
@Service
public class CubeTableStaticService extends ServiceImpl<CubeTableStaticMapper, CubeTableStatic> {

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private DynamicTableService dynamicTableService;

    @Autowired
    private DialectContext dialectContext;

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private CubeTableColumnService cubeTableColumnService;


    /**
     * @param dataParam
     * @return
     * @throws Exception
     */
    public CubeData doSubmit(CubeStaticDataParam dataParam) throws Exception {
        //1.识别表字段类型
        //2.创建表
        //3.存储静态文件表记录到业务库
        //4.存储上传的数据到物理表
        //5.创建数据集[该数据集不能显示]
        //6.创建关联的数据集字段
        List<List<String>> body = dataParam.getBody();
        //数据第一条记录为表头
        if (CollectionUtil.isEmpty(body) || body.size() <= 1) {
            throw new ServiceException("数据不能为空");
        }
        //获取数据源
        ReportDatasource datasource = reportDatasourceService.getByDsId(dataParam.getDatasourceId());
        if (datasource == null) {
            throw new ServiceException("当前代码行数："+Thread.currentThread().getStackTrace()[1].getLineNumber()+"数据源不存在");
        }
        //系统默认设置表名[表名注意大小写的问题]
        dataParam.setTableName("STATIC_TABLE_" + DateUtil.format(new Date(), "yyyyMMddHHmmss") + RandomUtil.randomNumbers(1));
        //如果存在cubeDataId，需要先删除一系列的内容
        _deleteCubeWithTable(dataParam.getCubeId());
        //先对数据做前置处理 ->> 删除所有null和空字符串的行
        body.removeIf(item -> item.stream().allMatch(StrUtil::isBlank));
        //表头
        Map<Integer, String> headDataMap = IntStream.range(0, body.get(0).size()).boxed().collect(Collectors.toMap(Function.identity(), i -> Optional.ofNullable(body.get(0).get(i)).orElse("")));
        //数据列表
        List<Map<Integer, String>> dataList = new ArrayList<Map<Integer, String>>();
        //跳过第一行
        body.stream().skip(1).forEach(item -> dataList.add(IntStream.range(0, item.size()).boxed().collect(Collectors.toMap(Function.identity(), i -> Optional.ofNullable(item.get(i)).orElse("")))));
        List<FileStructure> fileStructList = new ArrayList<>();
        headDataMap.forEach((k, v) -> {
            FileStructure fileStructure = new FileStructure();
            fileStructure.setColumnName(CubeConstants.COLUMN_NAME_PREFIX + k);
            fileStructure.setColumnType(CubeUtils.resolveDataType(k, CollectionUtil.sub(dataList, 0, 100)));
            fileStructure.setComment(v);
            fileStructure.setColumnIndex(k);
            fileStructList.add(fileStructure);
        });
        fileStructList.add(0, new FileStructure(CubeConstants.COLUMN_PRIMARY_KEY, ViewColumnTypeEnum.BIGINT, "主键ID", true));
        //(1)得到创建表SQL
        List<String> sqlList = dialectContext.getDialect(datasource.getDsTypeSyl()).buildCreateTableSql(dataParam.getTableName(), dataParam.getTableComment(), fileStructList);
        log.info("静态数据表sql:" + String.join(",", sqlList));
        //(2)执行SQL语句创建表
        dialectContext.getDialect(datasource.getDsTypeSyl()).executeSql(sqlList, datasource);
        log.info("创建静态数据表[{}]完成", dataParam.getTableName());
        //(3)上传数据到表
        dynamicTableService.batchSave(datasource, dataParam.getTableName(), SecureUtil.getUserId(), fileStructList.stream().toArray(FileStructure[]::new), dataList);
        CubeData cubeData = null;
        try {
            //4.创建数据集
            List<TableColumnDto> reportColumnList = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableColumnsByTableViewName(datasource, dataParam.getTableName());
            if (reportColumnList.isEmpty()) {
                throw new ServiceException("表[" + dataParam.getTableName() + "]不存在");
            }
            cubeData = saveCubeData(dataParam, reportColumnList, null,true);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            //如果创建数据集失败，删除无效的数据表，重新创建
            try {
                log.info("数据入库失败，删除静态文件表{}", dataParam.getTableName());
                dialectContext.getDialect(datasource.getDsTypeSyl()).executeSql(Arrays.asList("drop table " + dataParam.getTableName()), datasource);
            } catch (Exception ex) {
                log.error("删除数据表[" + dataParam.getTableName() + "]失败" + ex.getMessage(), ex);
            }
            throw e;
        }
        return cubeData;
    }

    /**
     * 临时数据集--零代码自动建表相关逻辑
     *
     * @param param
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public Map temporaryCubeData(TemporaryCubeDataParam param) throws Exception {
        //获取数据源
        ReportDatasource datasource = reportDatasourceService.getByDsId(param.getDatasourceId());
        if (datasource == null) {
            throw new ServiceException("当前代码行数："+Thread.currentThread().getStackTrace()[1].getLineNumber()+"数据源不存在");
        }
        List<CubeData> salveCubeDataList = new ArrayList<>();
        HashMap<String, Object> hashMap = new HashMap<>();
        hashMap.put("slaves", salveCubeDataList);
        for (TemporaryCubeDataParam.Table table : param.getTableList()) {
            boolean isMaster = true;
            //系统默认设置表名[表名注意大小写的问题]
            String tableName = "TABLE_" + DateUtil.format(new Date(), "yyyyMMddHHmmss") + RandomUtil.randomNumbers(1);
            ArrayList<FileStructure> fileStructList = new ArrayList<>();
            for (int i = 0; i < table.getColumnList().size(); i++) {
                TemporaryCubeDataParam.Column column = table.getColumnList().get(i);
                FileStructure fileStructure = new FileStructure();
                fileStructure.setColumnName(column.getName());
                fileStructure.setColumnType(column.getType());
                fileStructure.setComment(column.getComment());
                fileStructure.setColumnIndex(i);
                fileStructList.add(fileStructure);
                if (column.getType() == ViewColumnTypeEnum.BIGINT) {
                    isMaster = false;
                }
            }

            fileStructList.add(0, new FileStructure(CubeConstants.COLUMN_PRIMARY_KEY, ViewColumnTypeEnum.BIGINT, "主键ID", true));
            //(1)得到创建表SQL
            List<String> sqlList = dialectContext.getDialect(datasource.getDsTypeSyl()).buildCreateTableSql(tableName, table.getTableComment(), fileStructList);
            log.info("数据表sql:" + String.join(",", sqlList));
            //(2)执行SQL语句创建表
            dialectContext.getDialect(datasource.getDsTypeSyl()).executeSql(sqlList, datasource);
            log.info("创建数据表[{}]完成", tableName);
            CubeData cubeData = null;
            try {
                //4.创建数据集
                List<TableColumnDto> reportColumnList = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableColumnsByTableViewName(datasource, tableName);
                if (reportColumnList.isEmpty()) {
                    throw new ServiceException("表[" + tableName + "]不存在");
                }
                CubeStaticDataParam cubeStaticDataParam = new CubeStaticDataParam();
                cubeStaticDataParam.setCubeName(table.getTableComment());
                cubeStaticDataParam.setTableName(tableName);
                cubeStaticDataParam.setTableComment(table.getTableComment());
                cubeStaticDataParam.setDatasourceId(param.getDatasourceId());
                cubeData = saveCubeData(cubeStaticDataParam, reportColumnList, null, false);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                //如果创建数据集失败，删除无效的数据表，重新创建
                try {
                    log.info("数据入库失败，删除静态文件表{}", tableName);
                    dialectContext.getDialect(datasource.getDsTypeSyl()).executeSql(Arrays.asList("drop table " + tableName), datasource);
                } catch (Exception ex) {
                    log.error("删除数据表[" + tableName + "]失败" + ex.getMessage(), ex);
                }
                throw e;
            }
            if (isMaster) {
                hashMap.put("master", cubeData);
            } else {
                salveCubeDataList.add(cubeData);
            }
        }
        return hashMap;
    }

    /**
     * 删除内容包括：数据集，数据集字段，静态文件表记录，物理表
     *
     * @param cubeDataId
     * @throws Exception
     */
    @Transactional
    private void _deleteCubeWithTable(Integer cubeDataId) throws Exception {
        if (cubeDataId != null) {
            CubeTableStatic cubeTableStatic = getOne(Wrappers.<CubeTableStatic>lambdaQuery().eq(CubeTableStatic::getCubeDataId, cubeDataId), false);
            if (cubeTableStatic != null) {
                cubeDataService.removeById(cubeDataId);
                cubeTableColumnService.remove(Wrappers.<CubeTableColumn>lambdaQuery().eq(CubeTableColumn::getCubeDataId, cubeDataId));
                removeById(cubeTableStatic.getId());
                ReportDatasource datasource = reportDatasourceService.getByDsId(cubeTableStatic.getDatasourceId());
                if (datasource != null) {
                    String tableName = cubeTableStatic.getTableName();
                    dialectContext.getDialect(datasource.getDsTypeSyl()).executeSql(Arrays.asList("drop table " + tableName), datasource);
                }
            }
        }
    }

    @Transactional
    public CubeData saveCubeData(CubeStaticDataParam dataParam, List<TableColumnDto> reportColumnList, String cubeRemark, boolean whetherCreateCubeTableStatic) {
//        List<CubeTableColumn> cubeTableColumnList = new ArrayList<CubeTableColumn>();
//        int sortNum = 0;
        String tableAliasName = RandomUtil.randomString("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ", 5);
        List<CubeTableColumn> cubeTableColumnList = cubeTableColumnService.physicsTableColumnToCubeTableColumn(null, reportColumnList, dataParam.getTableName(), tableAliasName, 0);
//        for (TableColumnDto columnDto : reportColumnList) {
//            CubeTableColumn cubeTableColumn = new CubeTableColumn();
//            cubeTableColumn.setTableName(dataParam.getTableName());
//            cubeTableColumn.setAggregationType(AggregationTypeEnum.SUM);
//            cubeTableColumn.setColumnAttr(ColumnAttrEnum.NORMAL);
//            cubeTableColumn.setColumnComment(columnDto.getColumnComments());
//            cubeTableColumn.setColumnName(columnDto.getColumnName());
//            cubeTableColumn.setColumnTitle(StrUtil.isEmpty(columnDto.getColumnComments()) ? columnDto.getColumnName() : columnDto.getColumnComments());
//            cubeTableColumn.setViewDataType(columnDto.getColumnType());
//            cubeTableColumn.setOldViewDataType(columnDto.getColumnType());
//            cubeTableColumn.setDimensionType(columnDto.getColumnType() == ViewColumnTypeEnum.NUMBER ? DimensionTypeEnum.ZHIBIAO : DimensionTypeEnum.WEIDU);
//            cubeTableColumn.setHiddenStatus(0);
//            cubeTableColumn.setSortNum(sortNum);
//            cubeTableColumn.setTableAliasName(tableAliasName);
//            cubeTableColumn.setColumnSql(tableAliasName + "." + columnDto.getColumnName());
//            cubeTableColumn.setNumberDataFormat(NumberFormatEnum.AUTO);
//            cubeTableColumn.setTimeDataFormat(DateFormatEnum.yyyy_MM_ddhh$mm$ss);
//            cubeTableColumnList.add(cubeTableColumn);
//            sortNum += 100;
//        }
        CubeData cubeData = new CubeData();
        cubeData.setCubeName(dataParam.getCubeName());
        cubeData.setWorkspaceId(dataParam.getCubeWorkspaceId());
        cubeData.setDisplayStatus(0);
        cubeData.setCubeId(SnowFlowIntUtil.nextId());
        cubeData.setCubeTableNames(JSONArray.of(JSONObject.of("type", "table", "tableName", dataParam.getTableName(), "tableComment", dataParam.getTableComment(), "tableAliasName", tableAliasName)).toString());
        cubeData.setPublicStatus(0);
        cubeData.setDatasourceId(dataParam.getDatasourceId());
        cubeData.setQuerySql(dataParam.getTableName() + " " + tableAliasName);
        cubeData.setTableType(TableTypeEnum.PHYSICS);
        cubeData.setLinksJoin("[]");
        cubeData.setRemark(cubeRemark);
        if (cubeData.getId() == null) {
            cubeData.setCreateBy(SecureUtil.getNickName());
            cubeData.setCreateTime(new Date());
            cubeData.setCreateUserId(SecureUtil.getUserId());
        }
        //创建数据集和数据集字段明细
        R<Integer> ret = cubeDataService.doSaveOrUpdate(cubeData, cubeTableColumnList, null, AuthUtil.getUser());
        if (!ret.isSuccess()) {
            throw new ServiceException("数据集创建失败");
        }
        Integer cubeDataId = ret.getData();
        if (whetherCreateCubeTableStatic) {
            //存储静态文件表信息
            CubeTableStatic tableStatic = new CubeTableStatic();
            tableStatic.setCubeDataId(cubeDataId);
            tableStatic.setCreateBy(SecureUtil.getNickName());
            tableStatic.setCreateTime(new Date());
            tableStatic.setCreateUserId(SecureUtil.getUserId());
            tableStatic.setTableName(dataParam.getTableName());
            tableStatic.setTableComment(dataParam.getTableComment());
            tableStatic.setDatasourceId(dataParam.getDatasourceId());
            saveOrUpdate(tableStatic);
        }
        return cubeData;
    }
}
