/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.report.entity.Dimension;
import org.springblade.report.service.IDimensionService;
import org.springblade.report.vo.DimensionVO;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 报表维度 控制器
 *
 * @author BladeX
 * @since 2021-05-30
 */
@RestController
@AllArgsConstructor
@RequestMapping("report/dimension")
@Api(value = "报表维度", tags = "报表维度管理接口 DimensionController")
public class DimensionController extends BladeController {

	private final IDimensionService dimensionService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情 detail", notes = "传入dimension")
	public R<Dimension> detail(Dimension dimension) {
		Dimension detail = dimensionService.getOne(Condition.getQueryWrapper(dimension));
		return R.data(detail);
	}

	/**
	 * 分页 报表维度
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "列表 list", notes = "传入dimension")
	public R<List<Dimension>> list(Dimension dimension) {
		QueryWrapper queryWrapper = Condition.getQueryWrapper(dimension);
		queryWrapper.orderByAsc("order_index");
		List<Dimension> list = dimensionService.list(queryWrapper);
		return R.data(list);
	}

	/**
	 * 自定义分页 报表维度
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页 page", notes = "传入dimension")
	public R<IPage<DimensionVO>> page(DimensionVO dimension, Query query) {
		IPage<DimensionVO> pages = dimensionService.selectDimensionPage(Condition.getPage(query), dimension);
		return R.data(pages);
	}

	/**
	 * 新增 报表维度
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增 save", notes = "传入dimension")
	public R save(@Valid @RequestBody Dimension dimension) {
		if (null == dimension.getDatasourceId()) {
			return R.fail("数据源id不能为空");
		}
		return R.status(dimensionService.save(dimension));
	}

	/**
	 * 修改 报表维度
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改 update", notes = "传入dimension")
	public R update(@Valid @RequestBody Dimension dimension) {
		if (null == dimension.getDatasourceId()) {
			return R.fail("数据源id不能为空");
		}
		return R.status(dimensionService.updateById(dimension));
	}

	/**
	 * 新增或修改 报表维度
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改 submit", notes = "传入dimension")
	public R submit(@Valid @RequestBody Dimension dimension) {
		if (null == dimension.getDatasourceId()) {
			return R.fail("数据源id不能为空");
		}
		return R.status(dimensionService.saveOrUpdate(dimension));
	}


	/**
	 * 删除 报表维度
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "删除 remove", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(dimensionService.removeByIds(Func.toLongList(ids)));
	}

	/**
	 * 修改 报表维度
	 */
	@PostMapping("/batCreate")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "批量提交 batCreate", notes = "传入dimension")
	public R batCreate(@Valid @RequestBody List<Dimension> dimensionList) {
		if (null == dimensionList || dimensionList.size() == 0) {
			return R.fail("指标信息不能为空");
		}
		dimensionList.forEach(dimension -> {
			//查找是否存在
			QueryWrapper queryWrapper = new QueryWrapper();
			queryWrapper.eq("dimension_name", dimension.getDimensionName());
			queryWrapper.eq("table_view_name", dimension.getTableViewName());
			queryWrapper.eq("dimension_type", dimension.getDimensionType());
			List<Dimension> list = dimensionService.list(queryWrapper);
			if (null == list || list.size() == 0) {
				//不存在，则创建
				dimensionService.save(dimension);
			}
		});
		return R.success("批量提交完成");
	}


	/**
	 * 批量提交指标
	 */
	@PostMapping("/batSaveDimension")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "批量提交指标 batSaveDimension", notes = "传入dimension列表")
	public R batSaveDimension(@Valid @RequestBody List<Dimension> dimensionList) {
		if (null == dimensionList || dimensionList.size() == 0) {
			return R.fail("指标信息不能为空");
		}
/*		QueryWrapper wrapper = new QueryWrapper();
		wrapper.eq("table_view_name", dimensionList.get(0).getTableViewName());
		wrapper.eq("datasource_id", dimensionList.get(0).getDatasourceId());
		dimensionService.remove(wrapper);*/

		for (Dimension dimension : dimensionList) {
			//查找是否存在
			if (null == dimension.getDatasourceId()) {
				return R.fail("数据源id不能为空");
			}
			if (StringUtils.isEmpty(dimension.getTableViewName())) {
				return R.fail("数据表名不能为空");
			}
/*			dimension.setId(null);
			dimension.setCreateTime(null);*/
			dimensionService.saveOrUpdate(dimension);
		}
		return R.success("批量提交指标完成");
	}
}
