/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.controller;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.report.dto.request.QueryParam;
import org.springblade.report.dto.request.ReportApiQueryParam;
import org.springblade.report.dto.response.ReportInfoDTO;
import org.springblade.report.entity.ApiConfig;
import org.springblade.report.entity.ApiConfigDetail;
import org.springblade.report.entity.DataApi;
import org.springblade.report.entity.Dimension;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IApiConfigDetailService;
import org.springblade.report.service.IApiConfigService;
import org.springblade.report.service.IDataApiService;
import org.springblade.report.service.IDimensionService;
import org.springblade.report.service.IReportBaseService;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * 产品报表接口 控制器
 *
 * @author BladeX
 * @since 2021-03-10
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("report/reportApi")
@Api(value = "产品报表接口", tags = "报表型接口 ReportApiController")
public class ReportApiController extends BladeController {

	private IReportBaseService reportBaseService;
	private IDimensionService dimensionService;
	private final IDataApiService dataApiService;
	private final IReportDatasourceService reportDatasourceService;
//	private final DataSource dataSource;
//	private final DataSourceCreator dataSourceCreator;
	private final IApiConfigService apiConfigService;
	private final IApiConfigDetailService apiConfigDetailService;

	/**
	 * 获取报表视图
	 */
	@PostMapping("/getReportData")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "获取报表数据", notes = "")
	public R getReportData(@RequestBody QueryParam queryParam) {
		log.info("getReportData: {}", queryParam.toString());
		List<Integer> columnIdList = queryParam.getDimensionIdList();
		if (null == columnIdList || columnIdList.size() == 0) {
			return R.fail("字段id列表不可以为空");
		}
		if(queryParam.getDatasourceId() == null || queryParam.getDatasourceId() <= 0) {
			return R.fail("数据源ID不能为空");
		}
		//查询数据源
		ReportDatasource reportDatasource = reportDatasourceService.getByDsId(queryParam.getDatasourceId());
		if(reportDatasource == null) {
			return R.fail("数据源不存在，可能已删除。");
		}
		String poolName = reportDatasource.getPoolName();
		List<Dimension> dimensions = dimensionService.listByIds(columnIdList);
		if(dimensions == null || dimensions.isEmpty()) {
			return R.fail("没有查到维度数据");
		}
		List<String> selectSqlList = new ArrayList<String>();
		String tableName = "";
		for(Dimension dimension : dimensions) {
			if(StrUtil.isBlank(tableName)) {
				tableName = dimension.getTableViewName();
			}
			if (StrUtil.isBlank(dimension.getSqlText())) {
				selectSqlList.add(dimension.getDimensionName());
			} else {
				selectSqlList.add(dimension.getSqlText());
			}
		}
//		String tableName = dimensionService.getById(columnIdList.get(0)).getTableViewName();
		String limitSql = "";  //Limit Oracle/sqlserver不支持
		String selectSql = String.join(",", selectSqlList);
		if (null != queryParam.getLimitNum() && 
				reportDatasource.getDsType() != DsTypeEnum.oracle.getValue() && 
						reportDatasource.getDsType() != DsTypeEnum.sqlserver.getValue()) { 
			limitSql = "limit " + queryParam.getLimitNum();
		}
		
		List<LinkedHashMap<String, Object>> data = reportBaseService.getReportDataByCondition(poolName, tableName, selectSql, queryParam.getWhereSql(), queryParam.getOrderBySql(),
			queryParam.getGroupBySql(), limitSql);

		return R.data(data);
	}

	/**
	 * 操作符 0-等于 1-大于 2-大于等于 3-小于 4-小于等于 5-like 6-不等于
	 *
	 * @param operator
	 * @return
	 */
	String opeartorToString(Integer operator) {
		if (null == operator || 0 == operator || operator > 6) {
			return " = ";
		} else if (1 == operator) {
			return " &gt ";
		} else if (2 == operator) {
			return " &gt= ";
		} else if (3 == operator) {
			return " &lt= ";
		} else if (4 == operator) {
			return " &lt= ";
		} else if (5 == operator) {
			return " like ";
		} else if (6 == operator) {
			return " != ";
		}
		return "=";
	}


	@PostMapping("/{apiKey}")
	@ApiOperation(value = "执行报表类接口", notes = "传入apiKey")
	public R executeReportApi(@PathVariable("apiKey") String apiKey, HttpServletRequest request,
							  @RequestBody ReportApiQueryParam queryParam) {
		log.info("executeReportApi", apiKey);
		if (StrUtil.isBlank(apiKey)) {
			return R.fail("接口关键词不可以为空");
		}
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("api_key", apiKey);
		DataApi dataApi = dataApiService.getOne(queryWrapper);
		if (null == dataApi) {
			return R.fail("接口不存在，请检查接口关键词");
		}
		if (null == dataApi.getDatasourceId()) {
			return R.fail("接口未配置数据源");
		}
		if (0 != dataApi.getApiType()) {
			return R.fail("接口类型配置错误");
		}
		if (null != dataApi.getLoginRequired() && 1 == dataApi.getLoginRequired()) {
			BladeUser user = getUser();
			if (null == user) {
				return R.fail("未登录");
			}
		}
		Map<String, String[]> maps = request.getParameterMap();

		ReportDatasource datasource = reportDatasourceService.getByDsId(dataApi.getDatasourceId());
		if (null == datasource) {
			return R.fail("接口数据源配置错误");
		}
//		String poolName = datasource.getPoolName();
//		DynamicRoutingDataSource ds = (DynamicRoutingDataSource) dataSource;
//		Map<String, DataSource> dsMap = ds.getCurrentDataSources();
//		Boolean dsExist = false;
//		for (Map.Entry<String, DataSource> entry : dsMap.entrySet()) {
//			if (entry.getKey().equals(poolName)) {
//				dsExist = true;
//				break;
//			}
//		}
//		if (!dsExist) {
//			DataSourceProperty dataSourceProperty = new DataSourceProperty();
//			dataSourceProperty.setPoolName(datasource.getPoolName());
//			dataSourceProperty.setUrl(datasource.getUrl());
//			dataSourceProperty.setUsername(datasource.getDsUser());
//			dataSourceProperty.setPassword(datasource.getDsPassword());
//			dataSourceProperty.setDriverClassName(datasource.getDriverClassName());
//			DataSource dataSource = dataSourceCreator.createDataSource(dataSourceProperty);
//			ds.addDataSource(datasource.getPoolName(), dataSource);
//		}
//		HttpSession session = request.getSession();
//		session.setAttribute("tenantName", datasource.getPoolName());

		ReportInfoDTO reportInfoDTO = new ReportInfoDTO();
		QueryWrapper configQueryWrapper = new QueryWrapper();
		configQueryWrapper.eq("api_id", dataApi.getId());
		List<ApiConfig> configList = apiConfigService.list(configQueryWrapper);
		if (null == configList || configList.size() == 0) {
			return R.fail("接口配置错误");
		}
		List<Dimension> columnList = new ArrayList<>();
		List<Dimension> rowList = new ArrayList<>();
		List<Dimension> indexList = new ArrayList<>();
		for (ApiConfig apiConfig : configList) {
			if (apiConfig.getConfigType() == 1) {
				QueryWrapper configDetailWrapper = new QueryWrapper();
				configDetailWrapper.eq("api_config_id", apiConfig.getId());
				List<ApiConfigDetail> apiConfigDetailList = apiConfigDetailService.list(configDetailWrapper);
				apiConfigDetailList.forEach(item -> {
					Dimension dimension = dimensionService.getById(item.getDimensionId());
					if (null != dimension) {
						columnList.add(dimension);
					}
				});
			} else if (apiConfig.getConfigType() == 2) {
				QueryWrapper configDetailWrapper = new QueryWrapper();
				configDetailWrapper.eq("api_config_id", apiConfig.getId());
				List<ApiConfigDetail> apiConfigDetailList = apiConfigDetailService.list(configDetailWrapper);
				apiConfigDetailList.forEach(item -> {
					Dimension dimension = dimensionService.getById(item.getDimensionId());
					if (null != dimension) {
						rowList.add(dimension);
					}
				});
			} else if (apiConfig.getConfigType() == 3) {
				QueryWrapper configDetailWrapper = new QueryWrapper();
				configDetailWrapper.eq("api_config_id", apiConfig.getId());
				List<ApiConfigDetail> apiConfigDetailList = apiConfigDetailService.list(configDetailWrapper);
				apiConfigDetailList.forEach(item -> {
					Dimension dimension = dimensionService.getById(item.getDimensionId());
					if (null != dimension) {
						indexList.add(dimension);
					}
				});
			}
		}

		if (null == columnList || columnList.size() == 0) {
			return R.fail("列维度不能为空");
		}
		if (columnList.size() > 5) {
			return R.fail("列维度最多5层");
		}

		if (null == rowList || rowList.size() == 0) {
			return R.fail("行维度不能为空");
		}

		if (null == indexList || indexList.size() == 0) {
			return R.fail("指标列表不能为空");
		}

		List<String> columnNameList = new ArrayList<>();
		for (Dimension column : columnList) {
			column = dimensionService.getById(column.getId());
			if (null != column) {
				columnNameList.add(column.getDimensionName());
			}
		}
		List<String> rowNameList = new ArrayList<>();//行纬度字段
		String rowFieldsSql = "";//行子段
		List<String> rowHeaderList = new ArrayList<>();
		for (Dimension row : rowList) {
			row = dimensionService.getById(row.getId());
			if (null != row) {
				rowNameList.add(row.getDimensionName());
				if (StrUtil.isBlank(rowFieldsSql)) {
					rowFieldsSql = row.getDimensionName();
				} else {
					rowFieldsSql = rowFieldsSql + " , " + row.getDimensionName();
				}
				rowHeaderList.add(row.getDimensionComments());
			}
		}

		List<String> indexNameList = new ArrayList<>();//指标字段
		for (Dimension index : indexList) {
			index = dimensionService.getById(index.getId());
			if (null != index) {
				indexNameList.add(index.getDimensionName());
			}
		}
		StringBuilder fieldSql = new StringBuilder();
		StringBuilder conditionSql = new StringBuilder();
		for (String columnName : columnNameList) {
			if (StrUtil.isBlank(fieldSql)) {
				fieldSql.append(columnName);
			} else {
				fieldSql.append("," + columnName);
			}
			if (StrUtil.isBlank(conditionSql)) {
				conditionSql.append(columnName + " is not null");
			} else {
				conditionSql.append(" and " + columnName + " is not null");
			}
		}

/*		List<QueryConditionDTO> queryConditionList = queryParam.getQueryConditionList();
		String param1 = null, param2 = null, param3 = null, param4 = null, param5 = null, param6 = null, param7 = null, param8 = null, param9 = null, param10 = null;
		String param1Value = null, param2Value = null, param3Value = null, param4Value = null, param5Value = null, param6Value = null, param7Value = null, param8Value = null, param9Value = null, param10Value = null;
		if (null != queryConditionList && queryConditionList.size() > 0) {
			for (int k = 0; k < queryConditionList.size(); k++) {
				if (k == 0) {
					Integer id = queryConditionList.get(k).getId();
					Dimension dimension = dimensionService.getById(id);
					if (null != dimension && StrUtil.isNotBlank(queryConditionList.get(k).getValue())) {
						if (null == queryConditionList.get(k).getOperator()) {
							queryConditionList.get(k).setOperator(0);
						}
						param1 = dimension.getDimensionName() + opeartorToString(queryConditionList.get(k).getOperator());
					}
					if (5 == queryConditionList.get(k).getOperator()) {
						param1Value = "%" + queryConditionList.get(k).getValue() + "%";
					} else {
						param1Value = queryConditionList.get(k).getValue();
					}
				} else if (k == 1) {
					Integer id = queryConditionList.get(k).getId();
					Dimension dimension = dimensionService.getById(id);
					if (null != dimension && StrUtil.isNotBlank(queryConditionList.get(k).getValue())) {
						if (null == queryConditionList.get(k).getOperator()) {
							queryConditionList.get(k).setOperator(0);
						}
						param2 = dimension.getDimensionName() + opeartorToString(queryConditionList.get(k).getOperator());
					}
					if (5 == queryConditionList.get(k).getOperator()) {
						param2Value = "%" + queryConditionList.get(k).getValue() + "%";
					} else {
						param2Value = queryConditionList.get(k).getValue();
					}
				} else if (k == 2) {
					Integer id = queryConditionList.get(k).getId();
					Dimension dimension = dimensionService.getById(id);
					if (null != dimension && StrUtil.isNotBlank(queryConditionList.get(k).getValue())) {
						if (null == queryConditionList.get(k).getOperator()) {
							queryConditionList.get(k).setOperator(0);
						}
						param3 = dimension.getDimensionName() + opeartorToString(queryConditionList.get(k).getOperator());
					}
					if (5 == queryConditionList.get(k).getOperator()) {
						param3Value = "%" + queryConditionList.get(k).getValue() + "%";
					} else {
						param3Value = queryConditionList.get(k).getValue();
					}
				} else if (k == 3) {
					Integer id = queryConditionList.get(k).getId();
					Dimension dimension = dimensionService.getById(id);
					if (null != dimension && StrUtil.isNotBlank(queryConditionList.get(k).getValue())) {
						if (null == queryConditionList.get(k).getOperator()) {
							queryConditionList.get(k).setOperator(0);
						}
						param4 = dimension.getDimensionName() + opeartorToString(queryConditionList.get(k).getOperator());
					}
					if (5 == queryConditionList.get(k).getOperator()) {
						param4Value = "%" + queryConditionList.get(k).getValue() + "%";
					} else {
						param4Value = queryConditionList.get(k).getValue();
					}
				} else if (k == 4) {
					Integer id = queryConditionList.get(k).getId();
					Dimension dimension = dimensionService.getById(id);
					if (null != dimension && StrUtil.isNotBlank(queryConditionList.get(k).getValue())) {
						if (null == queryConditionList.get(k).getOperator()) {
							queryConditionList.get(k).setOperator(0);
						}
						param5 = dimension.getDimensionName() + opeartorToString(queryConditionList.get(k).getOperator());
					}
					if (5 == queryConditionList.get(k).getOperator()) {
						param5Value = "%" + queryConditionList.get(k).getValue() + "%";
					} else {
						param5Value = queryConditionList.get(k).getValue();
					}
				} else if (k == 5) {
					Integer id = queryConditionList.get(k).getId();
					Dimension dimension = dimensionService.getById(id);
					if (null != dimension && StrUtil.isNotBlank(queryConditionList.get(k).getValue())) {
						if (null == queryConditionList.get(k).getOperator()) {
							queryConditionList.get(k).setOperator(0);
						}
						param6 = dimension.getDimensionName() + opeartorToString(queryConditionList.get(k).getOperator());
					}
					if (5 == queryConditionList.get(k).getOperator()) {
						param6Value = "%" + queryConditionList.get(k).getValue() + "%";
					} else {
						param6Value = queryConditionList.get(k).getValue();
					}
				} else if (k == 6) {
					Integer id = queryConditionList.get(k).getId();
					Dimension dimension = dimensionService.getById(id);
					if (null != dimension && StrUtil.isNotBlank(queryConditionList.get(k).getValue())) {
						if (null == queryConditionList.get(k).getOperator()) {
							queryConditionList.get(k).setOperator(0);
						}
						param7 = dimension.getDimensionName() + opeartorToString(queryConditionList.get(k).getOperator());
					}
					if (5 == queryConditionList.get(k).getOperator()) {
						param7Value = "%" + queryConditionList.get(k).getValue() + "%";
					} else {
						param7Value = queryConditionList.get(k).getValue();
					}
				} else if (k == 7) {
					Integer id = queryConditionList.get(k).getId();
					Dimension dimension = dimensionService.getById(id);
					if (null != dimension && StrUtil.isNotBlank(queryConditionList.get(k).getValue())) {
						if (null == queryConditionList.get(k).getOperator()) {
							queryConditionList.get(k).setOperator(0);
						}
						param8 = dimension.getDimensionName() + opeartorToString(queryConditionList.get(k).getOperator());
					}
					if (5 == queryConditionList.get(k).getOperator()) {
						param8Value = "%" + queryConditionList.get(k).getValue() + "%";
					} else {
						param8Value = queryConditionList.get(k).getValue();
					}
				} else if (k == 8) {
					Integer id = queryConditionList.get(k).getId();
					Dimension dimension = dimensionService.getById(id);
					if (null != dimension && StrUtil.isNotBlank(queryConditionList.get(k).getValue())) {
						if (null == queryConditionList.get(k).getOperator()) {
							queryConditionList.get(k).setOperator(0);
						}
						param9 = dimension.getDimensionName() + opeartorToString(queryConditionList.get(k).getOperator());
					}
					if (5 == queryConditionList.get(k).getOperator()) {
						param9Value = "%" + queryConditionList.get(k).getValue() + "%";
					} else {
						param9Value = queryConditionList.get(k).getValue();
					}
				} else if (k == 9) {
					Integer id = queryConditionList.get(k).getId();
					Dimension dimension = dimensionService.getById(id);
					if (null != dimension && StrUtil.isNotBlank(queryConditionList.get(k).getValue())) {
						if (null == queryConditionList.get(k).getOperator()) {
							queryConditionList.get(k).setOperator(0);
						}
						param10 = dimension.getDimensionName() + opeartorToString(queryConditionList.get(k).getOperator());
					}
					if (5 == queryConditionList.get(k).getOperator()) {
						param10Value = "%" + queryConditionList.get(k).getValue() + "%";
					} else {
						param10Value = queryConditionList.get(k).getValue();
					}
				}
			}
		}*/

		List<LinkedHashMap<String, Object>> columnMapList = reportBaseService.selectRowToColList(dataApi.getQueryTable(),
			fieldSql.toString(), queryParam.getWhereSql());

		reportInfoDTO.setRowHeaderList(rowHeaderList);

		StringBuilder sb = new StringBuilder();
		List<String> header1 = new ArrayList<>();//第一行列维度
		List<String> header2 = new ArrayList<>();//第二行列维度
		List<String> header3 = new ArrayList<>();//第三行列维度
		List<String> header4 = new ArrayList<>();//第四行列维度
		List<String> header5 = new ArrayList<>();//第五行列维度
		List<String> header6 = new ArrayList<>();//第六行列维度 -- 指标
		Integer i = 0;
		for (Map<String, Object> map : columnMapList) {
			if (columnNameList.size() == 1) {
				//一层列维度， 算上指标，一共2层
				String firstLevelName = columnNameList.get(0);
				String firstLevelValue = map.get(firstLevelName).toString();
				for (int j = 0; j < indexNameList.size(); j++) {
					header1.add(firstLevelValue);
					header2.add(indexNameList.get(j));
					sb.append("SUM(CASE WHEN " + firstLevelName + "= '" + firstLevelValue + "'");
					String asName1 = "index" + i++;
					sb.append(" THEN " + indexNameList.get(j) + " ELSE 0 END) AS " + asName1 + ",");
				}
			} else if (columnNameList.size() == 2) {
				//二层列维度， 算上指标，一共3层
				String firstLevelName = columnNameList.get(0);
				String firstLevelValue = map.get(firstLevelName).toString();
				String secondLevelName = columnNameList.get(1);
				String secondLevelValue = map.get(secondLevelName).toString();
				for (int j = 0; j < indexNameList.size(); j++) {
					header1.add(firstLevelValue);
					header2.add(secondLevelValue);
					header3.add(indexNameList.get(j));
					sb.append("SUM(CASE WHEN " + secondLevelName + "= '" + secondLevelValue + "' AND " + firstLevelName + " = '" + firstLevelValue + "'");
					String asName1 = "index" + i++;
					sb.append(" THEN " + indexNameList.get(j) + " ELSE 0 END) AS " + asName1 + ",");
				}
			} else if (columnNameList.size() == 3) {
				//三层列维度， 算上指标，一共4层
				String firstLevelName = columnNameList.get(0);
				String firstLevelValue = map.get(firstLevelName).toString();
				String secondLevelName = columnNameList.get(1);
				String secondLevelValue = map.get(secondLevelName).toString();
				String thirdLevelName = columnNameList.get(2);
				String thirdLevelValue = map.get(thirdLevelName).toString();
				for (int j = 0; j < indexNameList.size(); j++) {
					header1.add(firstLevelValue);
					header2.add(secondLevelValue);
					header3.add(thirdLevelValue);
					header4.add(indexNameList.get(j));
					sb.append("SUM(CASE WHEN " + thirdLevelName + "= '" + thirdLevelValue + "' AND " + secondLevelName + "= '" + secondLevelValue + "' AND " + firstLevelName + " = '" + firstLevelValue + "'");
					String asName1 = "index" + i++;
					sb.append(" THEN " + indexNameList.get(j) + " ELSE 0 END) AS " + asName1 + ",");
				}
			} else if (columnNameList.size() == 4) {
				//四层列维度， 算上指标，一共5层
				String firstLevelName = columnNameList.get(0);
				String firstLevelValue = map.get(firstLevelName).toString();
				String secondLevelName = columnNameList.get(1);
				String secondLevelValue = map.get(secondLevelName).toString();
				String thirdLevelName = columnNameList.get(2);
				String thirdLevelValue = map.get(thirdLevelName).toString();
				String forthLevelName = columnNameList.get(3);
				String forthLevelValue = map.get(forthLevelName).toString();
				for (int j = 0; j < indexNameList.size(); j++) {
					header1.add(firstLevelValue);
					header2.add(secondLevelValue);
					header3.add(thirdLevelValue);
					header4.add(forthLevelValue);
					header5.add(indexNameList.get(j));
					sb.append("SUM(CASE WHEN " + forthLevelName + "= '" + forthLevelValue + "' AND " + thirdLevelName + "= '" + thirdLevelValue + "' AND " + secondLevelName + "= '" + secondLevelValue + "' AND " + firstLevelName + " = '" + firstLevelValue + "'");
					String asName1 = "index" + i++;
					sb.append(" THEN " + indexNameList.get(j) + " ELSE 0 END) AS " + asName1 + ",");
				}
			} else if (columnNameList.size() == 5) {
				//五层列维度， 算上指标，一共6层
				String firstLevelName = columnNameList.get(0);
				String firstLevelValue = map.get(firstLevelName).toString();
				String secondLevelName = columnNameList.get(1);
				String secondLevelValue = map.get(secondLevelName).toString();
				String thirdLevelName = columnNameList.get(2);
				String thirdLevelValue = map.get(thirdLevelName).toString();
				String forthLevelName = columnNameList.get(3);
				String forthLevelValue = map.get(forthLevelName).toString();
				String fifthLevelName = columnNameList.get(4);
				String fifthLevelValue = map.get(fifthLevelName).toString();
				for (int j = 0; j < indexNameList.size(); j++) {
					header1.add(firstLevelValue);
					header2.add(secondLevelValue);
					header3.add(thirdLevelValue);
					header4.add(forthLevelValue);
					header5.add(fifthLevelValue);
					header6.add(indexNameList.get(j));
					sb.append("SUM(CASE WHEN " + fifthLevelName + "= '" + fifthLevelValue + "' AND " + forthLevelName + "= '" + forthLevelValue + "' AND " + thirdLevelName + "= '" + thirdLevelValue + "' AND " + secondLevelName + "= '" + secondLevelValue + "' AND " + firstLevelName + " = '" + firstLevelValue + "'");
					String asName1 = "index" + i++;
					sb.append(" THEN " + indexNameList.get(j) + " ELSE 0 END) AS " + asName1 + ",");
				}
			}

		}
		String sql = sb.subSequence(0, sb.length() - 1).toString();
		System.out.println(sql);

		List<List<String>> headerList = new ArrayList<>();
		if (columnNameList.size() == 1) {
			headerList.add(header1);
			headerList.add(header2);
		} else if (columnNameList.size() == 2) {
			headerList.add(header1);
			headerList.add(header2);
			headerList.add(header3);
		} else if (columnNameList.size() == 3) {
			headerList.add(header1);
			headerList.add(header2);
			headerList.add(header3);
			headerList.add(header4);
		} else if (columnNameList.size() == 4) {
			headerList.add(header1);
			headerList.add(header2);
			headerList.add(header3);
			headerList.add(header4);
			headerList.add(header5);
		} else if (columnNameList.size() == 5) {
			headerList.add(header1);
			headerList.add(header2);
			headerList.add(header3);
			headerList.add(header4);
			headerList.add(header5);
			headerList.add(header6);
		}

		reportInfoDTO.setHeaderList(headerList);


		List<LinkedHashMap<String, Object>> data = reportBaseService.getReportData(dataApi.getQueryTable(), rowFieldsSql, sql, queryParam.getWhereSql(), queryParam.getOrderBySql());

		reportInfoDTO.setData(data);

		return R.data(reportInfoDTO);

	}

	Boolean checkSqlKeywords(String sql) {
		String str = sql.toLowerCase();
		if (str.contains("insert") || str.contains("update") || str.contains("delete") || str.contains("drop") || str.contains("alter") || str.contains("set")) {
			return true;
		}
		return false;
	}
}
