/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.controller;

import java.util.List;

import javax.validation.Valid;

import org.springblade.bigdata.entity.Visual;
import org.springblade.bigdata.entity.VisualConfig;
import org.springblade.bigdata.service.IVisualConfigService;
import org.springblade.bigdata.service.IVisualService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.report.dto.request.AddTemplateToReportRequest;
import org.springblade.report.dto.request.CreateSubReportRequest;
import org.springblade.report.dto.request.SetReportRoleRequest;
import org.springblade.report.entity.Report;
import org.springblade.report.entity.ReportRole;
import org.springblade.report.entity.ReportTemplate;
import org.springblade.report.service.IReportRoleService;
import org.springblade.report.service.IReportService;
import org.springblade.report.service.IReportTemplateService;
import org.springblade.report.vo.ReportTemplateInfoVO;
import org.springblade.report.vo.ReportVO;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.bean.BeanUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * 报告 控制器
 *
 * @author BladeX
 * @since 2021-03-19
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("report/report")
@Api(value = "报告管理", tags = "报告管理 ReportController")
public class ReportController extends BladeController {

	private final IReportService reportService;
	private final IReportRoleService reportRoleService;
	private final IReportTemplateService reportTemplateService;
	private final IVisualService visualService;
	private final IVisualConfigService visualConfigService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情 detail", notes = "传入report")
	public R<Report> detail(Report report) {
		Report detail = reportService.getOne(Condition.getQueryWrapper(report));
		return R.data(detail);
	}

	/**
	 * 分页 报告
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "列表 list", notes = "传入report")
	public R<List<Report>> list(Report report) {
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.like("title", report.getTitle());
		List<Report> list = reportService.list(queryWrapper);
		return R.data(list);
	}

	/**
	 * 自定义分页 报告
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页 page", notes = "传入report")
	public R<IPage<ReportVO>> page(ReportVO report, Query query) {
		IPage<ReportVO> pages = reportService.selectReportPage(Condition.getPage(query), report);
		return R.data(pages);
	}

	/**
	 * 新增 报告
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增 save", notes = "传入report")
	public R save(@Valid @RequestBody ReportVO reportVO) {
		Report report = reportVO;
		report.setCreateUser(getUser().getUserId());
		Boolean rst = reportService.save(report);
		if (rst) {
			reportVO.getReadRoleList().forEach(role -> {
				ReportRole reportRole = new ReportRole();
				reportRole.setReportId(report.getId());
				reportRole.setRoleId(role.getId());
				reportRole.setType(1);
				reportRoleService.save(reportRole);
			});
			reportVO.getEditRoleList().forEach(roleId -> {
				ReportRole reportRole = new ReportRole();
				reportRole.setReportId(report.getId());
				reportRole.setRoleId(roleId.getId());
				reportRole.setType(2);
				reportRoleService.save(reportRole);
			});
		}
		return R.status(rst);
	}

	/**
	 * 修改 报告
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改 update", notes = "传入report")
	public R update(@Valid @RequestBody ReportVO reportVO) {
		Report report = reportVO;
		report.setUpdateUser(getUser().getUserId());
		Boolean rst = reportService.updateById(report);
		if (rst) {
			QueryWrapper queryWrapper = new QueryWrapper();
			queryWrapper.eq("report_id", report.getId());
			reportRoleService.remove(queryWrapper);
			reportVO.getReadRoleList().forEach(role -> {
				ReportRole reportRole = new ReportRole();
				reportRole.setReportId(report.getId());
				reportRole.setRoleId(role.getId());
				reportRole.setType(1);
				reportRoleService.save(reportRole);
			});
			reportVO.getEditRoleList().forEach(roleId -> {
				ReportRole reportRole = new ReportRole();
				reportRole.setReportId(report.getId());
				reportRole.setRoleId(roleId.getId());
				reportRole.setType(2);
				reportRoleService.save(reportRole);
			});
		}
		return R.status(rst);
	}

	/**
	 * 删除 报告
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "删除 remove", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(reportService.removeByIds(Func.toLongList(ids)));
	}

	/**
	 * 设置报告权限
	 */
	@PostMapping("/setReportRole")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "单独设置报告权限", notes = "单独设置报告权限")
	public R setReportRole(@RequestBody SetReportRoleRequest setReportRoleRequest) {
		Integer reportId = setReportRoleRequest.getReportId();
		List<Long> readRoleIdList = setReportRoleRequest.getReadRoleIdList();
		List<Long> editRoleIdList = setReportRoleRequest.getEditRoleIdList();
		if (null == reportId) {
			return R.fail("报告id不可以为空");
		}
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("report_id", reportId);
		boolean removeResult = reportRoleService.remove(queryWrapper);
		readRoleIdList.forEach(roleId -> {
			ReportRole reportRole = new ReportRole();
			reportRole.setReportId(reportId);
			reportRole.setRoleId(roleId);
			reportRole.setType(1);
			reportRoleService.save(reportRole);
		});
		editRoleIdList.forEach(roleId -> {
			ReportRole reportRole = new ReportRole();
			reportRole.setReportId(reportId);
			reportRole.setRoleId(roleId);
			reportRole.setType(2);
			reportRoleService.save(reportRole);
		});

		return R.status(removeResult);
	}

	/**
	 * 添加模板到报告
	 */
	@PostMapping("/addTempateToReport")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "添加模板到报告 addTempateToReport", notes = "传入reportId和模板列表")
	public R addTempateToReport(@Valid @RequestBody AddTemplateToReportRequest addTemplateToReportRequest) {
		log.info("addTempateToReport", addTemplateToReportRequest.toString());
		Integer reportId = addTemplateToReportRequest.getReportId();
		Report report = reportService.getById(reportId);
		if (null == report) {
			return R.fail("报告不存在");
		}
		QueryWrapper templateWrapper = new QueryWrapper();
		templateWrapper.eq("report_id", reportId);
		reportTemplateService.remove(templateWrapper);
		List<Long> templateIdList = addTemplateToReportRequest.getTemplateIdList();
		templateIdList.forEach(templateId -> {
			Visual visual = visualService.getById(templateId);
			if (null != visual) {
				ReportTemplate reportTemplate = new ReportTemplate();
				reportTemplate.setReportId(reportId);
				reportTemplate.setTitle(visual.getTitle());
				reportTemplate.setOrigTemplateId(visual.getId());
				QueryWrapper queryWrapper = new QueryWrapper();
				queryWrapper.eq("visual_id", visual.getId());
				VisualConfig visualConfig = visualConfigService.getOne(queryWrapper);
				if (null != visualConfig) {
					reportTemplate.setDetail(visualConfig.getDetail());
					reportTemplate.setComponent(visualConfig.getComponent());
				}
				reportTemplateService.save(reportTemplate);
			}
		});

		return R.status(true);
	}


	/**
	 * 编辑报告模板
	 */
	@PostMapping("/modifyReportTemplate")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "编辑报告模板 modifyReportTemplate", notes = "传入reportId和模板信息")
	public R modifyReportTemplate(@Valid @RequestBody ReportTemplateInfoVO reportTemplateInfoVO) {
		log.info("modifyReportTemplate", reportTemplateInfoVO.toString());
		Report report = reportService.getById(reportTemplateInfoVO.getId());
		if (null == report) {
			return R.fail("报告不存在");
		}
		QueryWrapper templateWrapper = new QueryWrapper();
		templateWrapper.eq("report_id", report.getId());
		reportTemplateService.remove(templateWrapper);
		List<ReportTemplate> reportTemplateList = reportTemplateInfoVO.getReportTemplateList();
		reportTemplateList.forEach(reportTemplate -> {
			reportTemplateService.save(reportTemplate);
		});
		return R.status(true);
	}

	/**
	 * 报告模板详情
	 */
	@GetMapping("/reportTemplateDetail")
	@ApiOperationSupport(order = 10)
	@ApiOperation(value = "详情 reportTemplateDetail", notes = "传入report id")
	public R<ReportTemplateInfoVO> reportTemplateDetail(@RequestParam(value = "id") Integer id) {
		if (null == id) {
			return R.fail("id错误");
		}
		Report report = reportService.getById(id);
		if (null == report) {
			return R.fail("报告不存在");
		}
		ReportTemplateInfoVO data = new ReportTemplateInfoVO();
		BeanUtil.copyProperties(report, data);
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("report_id", report.getId());
		data.setReportTemplateList(reportTemplateService.list(queryWrapper));
		return R.data(data);
	}

	/**
	 * 创建分期报告
	 */
	@PostMapping("/createSubReport")
	@ApiOperationSupport(order = 11)
	@ApiOperation(value = "创建分期报告 createSubReport", notes = "传入reportId和模板信息")
	public R createSubReport(@Valid @RequestBody CreateSubReportRequest createSubReportRequest) {
		log.info("createSubReport", createSubReportRequest.toString());
		Integer reportId = createSubReportRequest.getReportId();
		if (null == reportId) {
			return R.fail("id错误");
		}
		Report report = reportService.getById(reportId);
		if (null == report) {
			return R.fail("报告不存在");
		}
		Report subReport = new Report();
		BeanUtil.copyProperties(report, subReport);
		subReport.setId(null);
		subReport.setParentId(reportId);
		subReport.setCreateTime(null);
		if (StringUtil.isNotBlank(createSubReportRequest.getTitle())) {
			subReport.setTitle(createSubReportRequest.getTitle());
		} else {
			subReport.setTitle(report.getTitle());
		}
		reportService.save(subReport);
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("report_id", reportId);
		List<ReportTemplate> reportTemplateList = reportTemplateService.list(queryWrapper);
		reportTemplateList.forEach(reportTemplate -> {
			ReportTemplate subReportTemplate = new ReportTemplate();
			BeanUtil.copyProperties(reportTemplate, subReportTemplate);
			subReportTemplate.setId(null);
			subReportTemplate.setReportId(subReport.getId());
			reportTemplateService.save(subReportTemplate);
		});
		return R.status(true);
	}

	/**
	 * 根据报告id获取分期报告
	 */
	@GetMapping("/getSubReportList")
	@ApiOperationSupport(order = 12)
	@ApiOperation(value = "获取报告的分期报告列表", notes = "传入report id")
	public R<List<Report>> getSubReportList(@RequestParam(value = "id") Integer id) {
		if (null == id) {
			return R.fail("id错误");
		}
		Report report = reportService.getById(id);
		if (null == report) {
			return R.fail("报告不存在");
		}

		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("parent_id", id);
		return R.data(reportService.list(queryWrapper));
	}


}
