/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.visual.controller;

import java.util.Date;
import java.util.List;

import javax.validation.Valid;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.tool.api.R;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.entity.VisualClassify;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.service.VisualClassifyService;
import org.springblade.visual.service.VisualDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 可视化分类表 控制器
 *
 * @author BladeX
 * @since 2021-03-05
 */
@RestController
@RequestMapping("visual/classify")
@Api(value = "大屏分类", tags = "大屏分类接口")
public class VisualClassifyController extends BladeController {

	@Autowired
	private VisualClassifyService visualCategoryService;
	
	@Autowired
	private VisualDataService visualDataService;
	
	@Autowired
	private DataPermClient dataPermClient;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入visualCategory")
	public R<VisualClassify> detail(VisualClassify visualCategory) {
		VisualClassify detail = visualCategoryService.getOne(Condition.getQueryWrapper(visualCategory));
		return R.data(detail);
	}

	/**
	 * 分页 可视化分类表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入visualCategory")
	public R<List<VisualClassify>> list(VisualClassify visualCategory) {
		QueryWrapper<VisualClassify> queryWrapper = Condition.getQueryWrapper(visualCategory);
		long currUserId = getUser().getUserId();
		List<VisualClassify> dataList = visualCategoryService.list(queryWrapper.lambda().orderByDesc(VisualClassify::getSortNum));
		Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(currUserId);
		if(checkSuperPermRet != null && checkSuperPermRet) { //是否有超级权限
			return R.data(dataList);
		}else {
			//查询所有大屏有权限的分类ID集合
			List<Integer> classifyIdList = visualDataService.classifyIdListPerm(currUserId,0);
			//1.获取这些分类的父级ID[包含自己创建的分类父级别ID]
			dataList.stream().filter(f -> (classifyIdList.contains(f.getId()) || f.getCreateUserId().longValue() == currUserId) && f.getParentId() != 0).map(VisualClassify::getParentId).forEach(classifyIdList::add);
			//2.删除不在权限内的分类[且createUserId不是本人的]
			dataList.removeIf(y -> !classifyIdList.contains(y.getId()) && y.getCreateUserId().longValue() != currUserId);
			return R.data(dataList);
		}
	}

	/**
	 * 新增或修改 可视化分类表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入visualCategory")
	public R<String> submit(@Valid @RequestBody VisualClassify visualCategory) {
		if(visualCategory.getId() == null && visualCategory.getParentId() != null && visualCategory.getParentId() > 0) {
			//添加子级分类，判断父级分类是否有大屏模版
			long visualCount = visualDataService.count(Wrappers.<VisualData>lambdaQuery()
					.eq(VisualData::getCategory, visualCategory.getParentId())
					.eq(VisualData::getVisualDataType,0));
			if(visualCount > 0) {
				return R.fail("你选择的父级分类下有大屏模版，不能新建子分类。");
			}
		}
		if(visualCategory.getId() == null) {
			visualCategory.setCreateUserId(getUser().getUserId());
			visualCategory.setCreateTime(new Date());
		}
		return R.status(visualCategoryService.saveOrUpdate(visualCategory));
	}
	
	/**
	 * 删除 可视化分类表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "根据ID删除", notes = "传入id")
	public R<String> remove(@ApiParam(value = "主键", required = true) @RequestParam Integer id) {
		long count = visualCategoryService.count(Wrappers.<VisualClassify>lambdaQuery().eq(VisualClassify::getParentId, id));
		if(count > 0) {
			return R.fail("该分类下有子分类，不能删除。");
		}
		long visualCount = visualDataService.count(Wrappers.<VisualData>lambdaQuery()
				.eq(VisualData::getCategory, id)
				.eq(VisualData::getVisualDataType,0));
		if(visualCount > 0) {
			return R.fail("该分类下有大屏模版，不能删除。");
		}
		return R.status(visualCategoryService.removeById(id));
	}

	@ApiOperation("根据租户id查询大屏分类")
	@GetMapping("/list-tenantid")
	public R<List<VisualClassify>> getClassifyTenantId(String tenantId){

		return R.data(visualCategoryService.list(Wrappers.<VisualClassify>lambdaQuery()
				.eq(VisualClassify::getTenantId,tenantId)));
	}
}
