/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.visual.controller;

import java.time.LocalDateTime;
import java.util.List;

import javax.validation.Valid;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.visual.entity.VisualReportFolder;
import org.springblade.visual.entity.VisualReportInfo;
import org.springblade.visual.service.VisualReportFolderService;
import org.springblade.visual.service.VisualReportInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 历史报告文件夹 控制器
 *
 * @author BladeX
 * @since 2021-07-02
 */
@RestController
@RequestMapping("visual/reportfolder")
@Api(value = "报告文件夹", tags = "报告文件夹接口")
public class VisualReportFolderController extends BladeController {

	@Autowired
	private VisualReportFolderService visualReportFolderService;

	@Autowired
	private VisualReportInfoService reportInfoService;
	
	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入reportFolder")
	public R<VisualReportFolder> detail(VisualReportFolder reportFolder) {
		VisualReportFolder detail = visualReportFolderService.getOne(Condition.getQueryWrapper(reportFolder));
		return R.data(detail);
	}

	/**
	 * 分页 历史报告文件夹
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "列表", notes = "传入reportFolder")
	public R<List<VisualReportFolder>> list(VisualReportFolder reportFolder) {
		QueryWrapper<VisualReportFolder> queryWrapper = Condition.getQueryWrapper(reportFolder);
		queryWrapper.lambda().eq(VisualReportFolder::getUserId, getUser().getUserId())
		.orderByAsc(VisualReportFolder::getSortOrder).orderByDesc(VisualReportFolder::getId);
		return R.data(visualReportFolderService.list(queryWrapper));
	}

	/**
	 * 新增或修改 历史报告文件夹
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "新增或修改", notes = "传入reportFolder")
	public R<Integer> submit(@Valid @RequestBody VisualReportFolder reportFolder) {
		reportFolder.setUserId(getUser().getUserId());
		reportFolder.setCreateTime(LocalDateTime.now());
		//判断名称是否相同
		LambdaQueryWrapper<VisualReportFolder> queryWrapper = Wrappers.<VisualReportFolder>lambdaQuery().eq(VisualReportFolder::getFolderName, reportFolder.getFolderName())
					.eq(VisualReportFolder::getUserId, getUser().getUserId()).last("limit 1");
		VisualReportFolder dbReportFolder = visualReportFolderService.getOne(queryWrapper);
		if(dbReportFolder != null && ( reportFolder.getId() == null  //新增时，库里面已经存在相同用户相同名称的目录
				|| reportFolder.getId() != null && dbReportFolder.getId().intValue() != reportFolder.getId())) { //修改时，修改为相同的目录名称
			return R.fail("已存在相同的目录名称，请检查");
		}
		visualReportFolderService.saveOrUpdate(reportFolder);
		return R.data(reportFolder.getId());
	}


	/**
	 * 删除 历史报告文件夹
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R<String> remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		//判断是否有报告
		long count = reportInfoService.count(Wrappers.lambdaQuery(VisualReportInfo.class).in(VisualReportInfo::getFolderId, ids));
		if(count > 0) {
			return R.fail("该目录有报告，不能删除");
		}
		return R.status(visualReportFolderService.removeByIds(Func.toLongList(ids)));
	}

}
